/*
 * FileManager.h
 *
 *  Created on: 18/lug/2012
 *      Author: massimo
 */

#pragma once

#include <string>
#include <vector>

namespace mpeg7cdva
{

/**
 * @class FileManager
 * Helper class to manage lists of file names.
 * @author Massimo Balestri
 * @date 2012
 */
class FileManager {
private:
	std::vector<std::string> lines;
	std::string datasetBaseDir;
	std::string datasetName;
	std::string workspaceDir;
	size_t ds_size;				// number of elements in the dataset
	int irlevel;				// include recursion level
	bool hasWorkspaceDir;

	/**
	 * Read the list of images from the given annotation file.
	 * @param filepathname the pathname of the annotation text file containing the list of images.
	 * @return the number of lines read from the filename.
	 */
	size_t readAnnotation(const char * filepathname);

	/**
	 * Append another annotation to the current one.
	 * @param line the include string
	 */
	void appendAnnotation(std::string line);

public:

	/**
	 * Read the list of images from the given annotation file.
	 * @param annotationpathname the pathname of the annotation text file containing the list of images.
	 * @param level the recursion level (to avoid infinite loops); must be zero when called the first time.
	 */
	FileManager(const char * annotationpathname, int level = 0);

	virtual ~FileManager();

	/**
	 * Get the dataset base directory.
	 * @return the dataset path
	 */
	std::string getDatasetPath() const;

	/**
	 * Get the dataset name.
	 * @return the dataset name
	 */
	std::string getDatasetName() const;

	/**
	 * Get the dataset full pathname.
	 * @return the dataset pathname
	 */
	std::string getDatasetPathName() const;

	/**
	 * Set the workspace directory. This is the directory where output files will be stored.
	 * @param workdir the workspace directory
	 */
	void setWorkspaceDir(const char * workdir);

	/**
	 * Get the workspace directory. This is the directory where output files will be stored.
	 * @return the workspace directory
	 */
	std::string getWorkspaceDir() const;


	/**
	 * Get the dataset size.
	 * @return the number of lines read from the filename.
	 */
	size_t getDatasetSize() const;

	/**
	 * Convert a pathname into a pathname with the given extension.
	 * @param imageName the original image name;
	 * @param ext new extension;
	 * @return the modified pathname.
	 */
	std::string replaceExt(const std::string & imageName, const char * ext) const;

	/**
	 * Convert a pathname into a pathname with the given new path.
	 * @param imageName the original image name;
	 * @param newpath the new path;
	 * @return the modified pathname.
	 */
	static std::string replacePath(const std::string & imageName, const char * newpath);

	/**
	 * Get the first image name found at the i-th position in the annotation file.
	 * The image name is provided as an absolute pathname.
	 * @param i the index of the image in the annotation file.
	 * @param absolutePathname if true, the absolute pathname of the file is returned; otherwise, the relative pathname is returned.
	 * @return the relative or absolute pathname of the image.
	 */
	std::string getQueryName(size_t i, bool absolutePathname = true) const;

	/**
	 * Get the second image name found at the i-th position in the annotation file.
	 * The image name is provided as an absolute pathname.
	 * @param i the index of the image in the annotation file.
	 * @param absolutePathname if true, the absolute pathname of the file is returned; otherwise, the relative pathname is returned.
	 * @return the relative or absolute pathname of the image.
	 */
	std::string getReferenceName(size_t i, bool absolutePathname = true) const;

	/**
	 * Count how many pathnames are contained in the i-th line.
	 * @param i the index of the image in the annotation file.
	 * @return the number of pathnames found.
	 */
	int countNames(size_t i) const;
};

}	// end namespace
